@@ -18,15 +18,14 @@ var supertest = require('supertest');
 var testData = require('./support/test_data');
 
 var CONTEXT_PATH = '';
-var PREFIX = CONTEXT_PATH + '/file';
+var WORKSPACE_ID = "orionode";
+var PREFIX = CONTEXT_PATH + '/file/' + WORKSPACE_ID;
 var WORKSPACE = path.join(__dirname, '.test_workspace');
 
-var app = express()
-.use(/* @callback */ function(req, res, next) {
-	req.user = { workspaceDir: WORKSPACE };
-	next();
-})
-.use(PREFIX + "*", require('../lib/file')({root: '/file'}));
+var app = express();
+app.locals.metastore = require('../lib/metastore/fs/store')({workspaceDir: WORKSPACE});
+app.locals.metastore.setup(app);
+app.use(CONTEXT_PATH + '/file' + "*", require('../lib/file')({gitRoot: CONTEXT_PATH + '/gitapi', fileRoot: CONTEXT_PATH + '/file', workspaceRoot: CONTEXT_PATH + '/workspace'}));
 var request = supertest.bind(null, app);
 
 function byName(a, b) {
